@@ -20,18 +20,17 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: ClockTool.cc 4844 2007-04-08 01:59:41Z simonb $
-
 #include "ClockTool.hh"
 
 #include "ToolTheme.hh"
 #include "Screen.hh"
-#include "CommandParser.hh"
+#include "FbTk/CommandParser.hh"
 #include "CommandDialog.hh"
 #include "fluxbox.hh"
 
 #include "FbTk/SimpleCommand.hh"
 #include "FbTk/ImageControl.hh"
+#include "FbTk/TextUtils.hh"
 #include "FbTk/Menu.hh"
 #include "FbTk/MenuItem.hh"
 #include "FbTk/I18n.hh"
@@ -46,79 +45,101 @@
   #include <time.h>
 #endif
 #include <sys/time.h>
-#include <string>
 #include <typeinfo>
 
+
+namespace {
+
+static const char SWITCHES_SECONDS[] = "crsSTX+";
+static const char SWITCHES_12_24H[] = "lIrkHT";
+static const char SWITCHES_24_12H[] = "kHTlIr";
+static const char SWITCH_AM_PM[] = "pP";
+
+/**
+ * return true if clock shows seconds. If clock doesn't show seconds then
+ * there is no need to wake up every second to redraw the clock.
+ */
+
+int showSeconds(const std::string& fmt_string) {
+
+    return FbTk::StringUtil::findCharFromAlphabetAfterTrigger(
+        fmt_string, '%', SWITCHES_SECONDS, sizeof(SWITCHES_SECONDS), 0) != std::string::npos;
+}
+
+
+timeval calcNextTimeout(const std::string& fmt_string) {
+    timeval now;
+    timeval next;
+    gettimeofday(&now, 0);
+    next.tv_sec = 60 - (now.tv_sec % 60) - 1;
+    next.tv_usec = 1000000 - now.tv_usec;
+    if (next.tv_usec >= 1000000) {
+        next.tv_sec++;
+        next.tv_usec -= 1000000;
+    }
+
+    // wake up at next second-change
+    if (showSeconds(fmt_string)) {
+        next.tv_sec = 0;
+    }
+
+    return next;
+}
+
+
+} // end of anonymous namespace
+
+
 class ClockMenuItem: public FbTk::MenuItem {
 public:
     explicit ClockMenuItem(ClockTool &tool):
-        FbTk::MenuItem(""), m_tool(tool) { 
-        // determine 12/24 hour format
-        _FB_USES_NLS;
-        if (m_tool.timeFormat().find("%k") != std::string::npos ||
-            m_tool.timeFormat().find("%H") != std::string::npos ||
-            m_tool.timeFormat().find("%T") != std::string::npos)
-            setLabel( _FB_XTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
-        else
-            setLabel( _FB_XTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
+        FbTk::MenuItem(FbTk::BiDiString("")), m_tool(tool) {
+
+        setClockModeLabel();
+        setCloseOnClick(false);
     }
 
-    void click(int button, int time) {
-        std::string newformat = m_tool.timeFormat();
-        size_t pos = newformat.find("%k");
-        std::string newstr;
-        bool clock24hour = true;
+    void click(int button, int time, unsigned int mods) {
 
-        _FB_USES_NLS;
+        // does the current format string contain something with 24/12h?
+        size_t found;
+        size_t pos = FbTk::StringUtil::findCharFromAlphabetAfterTrigger(
+                m_tool.timeFormat(), '%', SWITCHES_24_12H, sizeof(SWITCHES_24_12H), &found);
 
-        if (pos != std::string::npos)
-            newstr = "%l";
-        else if ((pos = newformat.find("%H")) != std::string::npos)
-            newstr = "%I";
-        else if ((pos = newformat.find("%T")) != std::string::npos)
-            newstr = "%r";
-        
-        // 12 hour
-        if (newstr.empty()) {
-            clock24hour = false;
-            if ((pos = newformat.find("%l")) != std::string::npos)
-                newstr = "%k";
-            else if ((pos = newformat.find("%I")) != std::string::npos)
-                newstr = "%H";
-            else if ((pos = newformat.find("%r")) != std::string::npos)
-                newstr = "%T";
-            
-        }
-        
-        if (!newstr.empty()) {
+        if (pos != std::string::npos) { // if so, exchange it with 12/24h
+            std::string newformat = m_tool.timeFormat();
+            newformat[pos+1] = SWITCHES_12_24H[found];
 
-            newformat.replace(pos, 2, newstr);
-            if (!clock24hour) { // erase %P/%p (AM|PM / am|pm)
-                pos = newformat.find("%p");
-                if (pos != std::string::npos)
-                    newformat.erase(pos, 2);
-                else if ((pos = newformat.find("%P")) != std::string::npos)
+            if (found < 3) { // 24h? erase %P/%p (AM|PM / am|pm)
+                pos = FbTk::StringUtil::findCharFromAlphabetAfterTrigger(
+                    newformat, '%', SWITCH_AM_PM, sizeof(SWITCH_AM_PM), 0);
+                if (pos != std::string::npos) {
                     newformat.erase(pos, 2);
+                }
             }
 
-        
             m_tool.setTimeFormat(newformat);
+            setClockModeLabel();
 
-            if (m_tool.timeFormat().find("%k") != std::string::npos ||
-                m_tool.timeFormat().find("%H") != std::string::npos ||
-                m_tool.timeFormat().find("%T") != std::string::npos)
-                setLabel( _FB_XTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
-            else
-                setLabel( _FB_XTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
-        
         } // else some other strange format...so we don't do anything
-        FbTk::MenuItem::click(button, time);
+        FbTk::MenuItem::click(button, time, mods);
     }
 private:
+
+    void setClockModeLabel() {
+        _FB_USES_NLS;
+        if (FbTk::StringUtil::findCharFromAlphabetAfterTrigger(
+            m_tool.timeFormat(), '%', SWITCHES_24_12H, 3, 0) != std::string::npos) {
+            setLabel( _FB_XTEXT(Toolbar, Clock24,   "Clock: 24h",   "set Clockmode to 24h") );
+        } else {
+            setLabel( _FB_XTEXT(Toolbar, Clock12,   "Clock: 12h",   "set Clockmode to 12h") );
+        }
+    }
+
     ClockTool &m_tool;
 };
 
-class EditClockFormatCmd: public FbTk::Command {
+class EditClockFormatCmd: public FbTk::Command<void> {
 public:
     void execute() {
         BScreen *screen = Fluxbox::instance()->mouseScreen();
@@ -126,9 +147,9 @@ public:
             return;
         std::string resourcename = screen->name() + ".strftimeFormat";
 
-        CommandDialog *dialog = new CommandDialog(*screen, "Edit Clock Format", 
+        CommandDialog *dialog = new CommandDialog(*screen, "Edit Clock Format",
                                                   "SetResourceValue " + resourcename + " ");
-        FbTk::RefCount<FbTk::Command> cmd(CommandParser::instance().parseLine("reconfigure"));
+        FbTk::RefCount<FbTk::Command<void> > cmd(FbTk::CommandParser<void>::instance().parse("reconfigure"));
         dialog->setPostCommand(cmd);
         dialog->setText(screen->resourceManager().resourceValue(resourcename));
         dialog->show();
@@ -136,13 +157,14 @@ public:
 };
 
 ClockTool::ClockTool(const FbTk::FbWindow &parent,
-                     ToolTheme &theme, BScreen &screen, FbTk::Menu &menu):
+                     FbTk::ThemeProxy<ToolTheme> &theme, BScreen &screen,
+                     FbTk::Menu &menu):
     ToolbarItem(ToolbarItem::FIXED),
-    m_button(parent, theme.font(), ""),
+    m_button(parent, theme->font(), FbTk::BiDiString("")),
     m_theme(theme),
     m_screen(screen),
     m_pixmap(0),
-    m_timeformat(screen.resourceManager(), std::string("%k:%M"), 
+    m_timeformat(screen.resourceManager(), std::string("%k:%M"),
                  screen.name() + ".strftimeFormat", screen.altName() + ".StrftimeFormat"),
     m_stringconvertor(FbTk::StringConvertor::ToFbString) {
     // attach signals
@@ -157,23 +179,21 @@ ClockTool::ClockTool(const FbTk::FbWindow &parent,
 
     _FB_USES_NLS;
 
-    // setup timer to check the clock every 0.01 second
-    // if nothing has changed, it wont update the graphics
-    m_timer.setInterval(1);
-    // m_timer.setTimeout(delay); // don't need to set timeout on interval timer
-    FbTk::RefCount<FbTk::Command> update_graphic(new FbTk::SimpleCommand<ClockTool>(*this, 
+    m_timer.setTimeout(calcNextTimeout(*m_timeformat));
+
+    FbTk::RefCount<FbTk::Command<void> > update_graphic(new FbTk::SimpleCommand<ClockTool>(*this,
                                                                                     &ClockTool::updateTime));
     m_timer.setCommand(update_graphic);
     m_timer.start();
 
-    m_button.setGC(m_theme.textGC());
+    m_button.setGC(m_theme->textGC());
 
     // setup menu
-    FbTk::RefCount<FbTk::Command> saverc(CommandParser::instance().parseLine("saverc"));
+    FbTk::RefCount<FbTk::Command<void> > saverc(FbTk::CommandParser<void>::instance().parse("saverc"));
     FbTk::MenuItem *item = new ClockMenuItem(*this);
     item->setCommand(saverc);
     menu.insert(item);
-    FbTk::RefCount<FbTk::Command> editformat_cmd(new EditClockFormatCmd());
+    FbTk::RefCount<FbTk::Command<void> > editformat_cmd(new EditClockFormatCmd());
     menu.insert(_FB_XTEXT(Toolbar, ClockEditFormat,   "Edit Clock Format",   "edit Clock Format") , editformat_cmd);
 
 
@@ -220,12 +240,12 @@ void ClockTool::update(FbTk::Subject *subj) {
     updateTime();
 
     // + 2 to make the entire text fit inside
-    // we only replace numbers with zeros because everything else should be 
+    // we only replace numbers with zeros because everything else should be
     // relatively static. If we replace all text with zeros then widths of
     // proportional fonts with some strftime formats will be considerably off.
-    std::string text(m_button.text());
+    FbTk::FbString text(m_button.text().logical());
 
-    int textlen = text.size();    
+    int textlen = text.size();
     for (int i=0; i < textlen; ++i) {
         if (isdigit(text[i])) // don't bother replacing zeros
             text[i] = '0';
@@ -235,19 +255,16 @@ void ClockTool::update(FbTk::Subject *subj) {
     unsigned int new_width = m_button.width();
     unsigned int new_height = m_button.height();
     translateSize(orientation(), new_width, new_height);
-    new_width = m_theme.font().textWidth(text.c_str(), text.size());
+    new_width = m_theme->font().textWidth(text.c_str(), text.size());
     translateSize(orientation(), new_width, new_height);
     if (new_width != m_button.width() || new_height != m_button.height()) {
         resize(new_width, new_height);
         resizeSig().notify();
     }
 
-    if (subj != 0 && typeid(*subj) == typeid(ToolTheme))
-        renderTheme(m_button.alpha());
-
 }
 
-unsigned int ClockTool::borderWidth() const { 
+unsigned int ClockTool::borderWidth() const {
     return m_button.borderWidth();
 }
 
@@ -265,6 +282,8 @@ void ClockTool::updateTime() {
     gettimeofday(&now, 0);
     time_t the_time = now.tv_sec;
 
+    m_timer.setTimeout(calcNextTimeout(*m_timeformat));
+
     if (the_time != -1) {
         char time_string[255];
         int time_string_len;
@@ -277,12 +296,12 @@ void ClockTool::updateTime() {
         if( time_string_len == 0)
             return;
         std::string text = m_stringconvertor.recode(time_string);
-        if (m_button.text() == text)
+        if (m_button.text().logical() == text)
             return;
 
         m_button.setText(text);
 
-        unsigned int new_width = m_theme.font().textWidth(time_string, time_string_len) + 2;
+        unsigned int new_width = m_theme->font().textWidth(time_string, time_string_len) + 2;
         if (new_width > m_button.width()) {
             resize(new_width, m_button.height());
             resizeSig().notify();
@@ -295,34 +314,34 @@ void ClockTool::updateTime() {
 
 // Just change things that affect the size
 void ClockTool::updateSizing() {
-    m_button.setBorderWidth(m_theme.border().width());
-    // resizes if new timeformat 
+    m_button.setBorderWidth(m_theme->border().width());
+    // resizes if new timeformat
     update(0);
 }
 
 void ClockTool::reRender() {
-    if (m_pixmap) 
+    if (m_pixmap)
         m_screen.imageControl().removeImage(m_pixmap);
 
-    if (m_theme.texture().usePixmap()) {
+    if (m_theme->texture().usePixmap()) {
         m_pixmap = m_screen.imageControl().renderImage(width(), height(),
-                                                       m_theme.texture(), orientation());
+                                                       m_theme->texture(), orientation());
         m_button.setBackgroundPixmap(m_pixmap);
     } else {
         m_pixmap = 0;
-        m_button.setBackgroundColor(m_theme.texture().color());
+        m_button.setBackgroundColor(m_theme->texture().color());
     }
 }
 
 
-void ClockTool::renderTheme(unsigned char alpha) {
+void ClockTool::renderTheme(int alpha) {
     m_button.setAlpha(alpha);
-    m_button.setJustify(m_theme.justify());
+    m_button.setJustify(m_theme->justify());
 
     reRender();
 
-    m_button.setBorderWidth(m_theme.border().width());
-    m_button.setBorderColor(m_theme.border().color());
+    m_button.setBorderWidth(m_theme->border().width());
+    m_button.setBorderColor(m_theme->border().color());
     m_button.clear();
 }
 
