@@ -1,5 +1,5 @@
 // FbRootWindow.cc
-// Copyright (c) 2003 - 2005 Henrik Kinnunen (fluxgen at fluxbox dot org)
+// Copyright (c) 2003 - 2006 Henrik Kinnunen (fluxgen at fluxbox dot org)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -19,11 +19,9 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: FbRootWindow.cc 3865 2005-01-24 18:34:57Z mathias $
-
 #include "FbRootWindow.hh"
-#include "App.hh"
 
+#include "FbTk/App.hh"
 #include <X11/Xutil.h>
 
 FbRootWindow::FbRootWindow(int screen_num):
@@ -62,7 +60,3 @@ FbRootWindow::FbRootWindow(int screen_num):
         m_colormap = DefaultColormap(disp, screen_num);
     }
 }
-
-void FbRootWindow::updateGeometry() {
-    FbTk::FbWindow::updateGeometry();
-}
